// auth.controller.ts
import bcrypt from "bcrypt";
import jwt from "jsonwebtoken";
import { Request, Response } from "express";
import { db } from "../db";
import { JwtPayload } from "../types";
import { sendEmail, verifyEmailSend } from "../services/email.service";
import { verifySmsSend } from "../services/sms.service";
import dayjs from "dayjs";
import { addChangeLog } from "../helpers/changeLog.helper";
import { PrismaClient } from "@prisma/client";

const REFRESH_TOKEN_SECRET = process.env.REFRESH_TOKEN_SECRET || "";
const JWT_SECRET = process.env.JWT_SECRET || "";
const FRONT_URL = process.env.FRONT_URL || "";
const TOKEN_EXPIRY_MINUTES = 30;

const  prisma = new PrismaClient();

export const register = async (req: Request, res: Response) => {
  try {
    const {
      gender,
      email,
      password,
      username,
      dob,
      phoneNumber,
      firstName,
      middleName,
      designation,
      lastName,
      country,
    } = req.body;

    let error = false;
    let errorMessage = {
      email: "",
      username: "",
      phoneNumber: "",
    };

    const extUserEmail = await db.users.findFirst({
      where: {
        email: email.toLowerCase(),
      },
    });
    if (extUserEmail) {
      error = true;
      errorMessage.email = "Email already exist.";
    }
    const extUserPhone = await db.users.findFirst({
      where: {
        phoneNumber: phoneNumber.toString(),
      },
    });
    if (extUserPhone) {
      error = true;
      errorMessage.phoneNumber = "Phone number already exist.";
    }
    const extUserUsername = await db.users.findFirst({
      where: {
        username: username.toLowerCase(),
      },
    });
    if (extUserUsername) {
      error = true;
      errorMessage.username = "Username already exist.";
    }

    if (error) {
      return res.status(400).json({
        success: false,
        message:
          "User already exist, please try again with another email/username.",
        error: errorMessage,
      });
    }

    const hashedPassword = await bcrypt.hash(password, 10);

    const saveData = await db.users.create({
      data: {
        username: username.toLowerCase(),
        firstName,
        middleName,
        lastName,
        designation,
        phoneNumber: phoneNumber.toString(),
        email: email.toLowerCase(),
        gender,
        country,
        dob: new Date(dob),
        password: hashedPassword,
      },
    });

    res.status(201).json({
      success: true,
      message: "User created successfully",
      userId: saveData.id,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

export const registerChange = async (req: Request, res: Response) => {
  try {
    const { userId, email, phoneNumber } = req.body;

    const extUser = await db.users.findUnique({
      where: {
        id: userId,
      },
    });

    if (!extUser) {
      return res.status(400).json({
        success: false,
        message: "User not found",
      });
    }
    const saveData = await db.users.update({
      where: {
        id: extUser.id,
      },
      data: {
        email: email,
        phoneNumber: phoneNumber.toString(),
      },
    });

    res.status(200).json({
      success: true,
      message: "User update successfully",
      userId: saveData.id,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

export const login = async (req: Request, res: Response) => {
  try {
    const { email, password ,fcmToken,deviceType,deviceVersion} = req.body;
    const user = await db.users.findUnique({
      where: { email },
    });

    if (!user) {
      return res
        .status(401)
        .json({ success: false, message: "email not found", details: { path: "email" } });
    }

    if (!(await bcrypt.compare(password, user.password))) {
      return res
        .status(401)
        .send({ success: false, message: "Invalid Password", details: { path: "password" } });
    }

    const jwtPayload: JwtPayload = {
      userId: user.id,
      username: user.username,
      email: user.email,
      // businessRole: user.businessRole,
      // country: user.country,
      firstName: user.firstName,
      lastName: user.lastName,
      middleName: user.middleName || "",
      imageUrl: user.imageUrl,
      role: user.role,
    };

    const refreshToken = jwt.sign(jwtPayload, REFRESH_TOKEN_SECRET, {
      expiresIn: "30d",
    });
    const sessionToken = jwt.sign(jwtPayload, JWT_SECRET, {
      expiresIn: "2h",
    });
    const lastLoginAt = new Date()
    const userInfo = {
      id: user.id,
      username: user.username,
      firstName: user.firstName,
      middleName: user.middleName, // optional
      lastName: user.lastName,
      gender: user.gender,
      email: user.email,
      designation: user.designation,
      lastLoginAt:lastLoginAt,
      phoneNumber: user.phoneNumber, // 9876543212,
      dob: dayjs(user.dob).format("YYYY-MM-DD"), // YYYY-MM-DD only accept
      country: user.country,
      role: user.role,
      // businessRole: user.businessRole,
    };

    // Search if email verify or not
    const verifyEmail = await db.verifyEmail.findFirst({
      where: {
        email: user.email,
        UserId: user.id,
        isVerify: true,
      },
    });

    // Search if phone number verify or not
    let countryCode = "+1";
    switch (user.country) {
      case "IND":
        countryCode = "+91";
        break;
      case "USA":
        countryCode = "+1";
        break;
    }
    const verifyPhone = await db.verifyPhone.findFirst({
      where: {
        phone: `${countryCode} ${user.phoneNumber}`,
        UserId: user.id,
        isVerify: true,
      },
    });

    const fcmFound = user.fcmTokens.find((item)=>item===fcmToken);
    const updatedFcmTokens = fcmFound ?  user.fcmTokens:[...user.fcmTokens, fcmToken];
    const cleanFcmTokens = updatedFcmTokens.filter((token) => token !== undefined);

    await db.users.update({where:{id:user.id},data:{
      lastLoginAt:lastLoginAt,
      deviceType: deviceType,
      deviceVersion: deviceVersion,
      fcmTokens:cleanFcmTokens
    }})
    
    // Store refresh token in user device db
    await db.deviceInfo.create({
      data: {
        ip: "1234",
        refreshToken: refreshToken,
        description: "Login",
        deviceName: deviceType,
        deviceVersion: deviceVersion,
        fcmToken: fcmToken,
        UserId: user.id,
      },
    });

    addChangeLog({
      userId: user.id ,
      role: user?.role || "DEFAULT",
      module: "login",
      description: `You logged in`,
      oldValues: {},
      newValues: {},
    });
    // if login user is Admin user then business always true
    // const isBusiness = user.businessInfo.length > 0;
    // const isBusinessFlag = user.role === "ADMIN" ? true : isBusiness;

    res.status(200).json({
      success: true,
      message: "Login successful",
      sessionToken, // this token content user info
      refreshToken, // this token content user info
      userInfo, // passing user info for init form fill in general registration
      verifyPhone: verifyPhone !== null, // pass true if verify else false
      verifyEmail: verifyEmail !== null, // pass true if verify else false
      // business: isBusinessFlag,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

// export const login  = async (req: Request, res: Response) => {
//   try {
//     const {email,  password } =req.body;
//     const user = await prisma.users.findUnique({
//       where: { email },
//     });
//     console.log("user",user);
//     if (!user) {
//       return res
//         .status(401)
//         .json({ success: false, message: "Username not found", details: { path: "username" } });
//     }
//     if (!(await bcrypt.compare(password, user.password))) {
//       return res
//         .status(401)
//         .json({ success: false, message: "Invalid Password", details: { path: "password" } });
//     }
//     return res.status(200).json({ success: true, message: "Login successful" });
//   } catch (error) {
//     console.error(error);
//     res.status(500).json({ message: "Internal Server Error", success: false });
    
//   }
// };

export const emailVerifySend = async (req: Request, res: Response) => {
  try {
    const { email, type, userId } = req.body;

    const extUser = await db.users.findUnique({
      where: {
        email: email,
        AND: {
          id: {
            not: userId,
          },
        },
      },
    });

    if (extUser) {
      return res.status(400).json({
        success: false,
        message:
          "Email address already taken, please try again with another email address.",
      });
    }

    if (type === "NEW") {
      const otpRes = await verifyEmailSend(email, userId);
      return res.status(200).json({
        success: true,
        message: "OTP sent successfully",
        verifyId: otpRes.verifyId,
        otp: otpRes.otp,
      });
    } else if (type === "CHANGE") {
      const alreadyEmail = await db.verifyEmail.findFirst({
        where: {
          UserId: userId,
          email: email,
        },
      });
      if (!alreadyEmail || alreadyEmail.isVerify === false) {
        const otpRes = await verifyEmailSend(email, userId);
        return res.status(200).json({
          success: true,
          message: "OTP sent successfully",
          verifyId: otpRes.verifyId,
        });
      } else {
        return res
          .status(400)
          .json({ success: false, message: "Email already verified" });
      }
    }
    res
      .status(400)
      .json({ success: false, message: "Something wrong, pls try again" });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

export const phoneVerifySend = async (req: Request, res: Response) => {
  try {
    const { phoneNumber, type, userId } = req.body;

    const extUser = await db.users.findFirst({
      where: {
        phoneNumber: phoneNumber.toString(),
        AND: {
          id: {
            not: userId,
          },
        },
      },
    });

    if (extUser) {
      return res.status(400).json({
        success: false,
        message:
          "Phone number already taken, please try again with another phone number.",
      });
    }

    const currentUser = await db.users.findFirst({
      where: {
        id: userId,
      },
    });

    if (type === "NEW") {
      const otpRes = await verifySmsSend(
        phoneNumber,
        currentUser?.country || "USA",
        userId
      );
      return res.status(200).json({
        success: true,
        message: "OTP sent successfully",
        verifyId: otpRes.verifyId,
      });
    } else if (type === "CHANGE") {
      const alreadyPhone = await db.verifyPhone.findFirst({
        where: {
          UserId: userId,
          phone: phoneNumber,
        },
      });
      if (!alreadyPhone || alreadyPhone.isVerify === false) {
        const otpRes = await verifySmsSend(
          phoneNumber,
          currentUser?.country || "USA",
          userId
        );
        return res.status(200).json({
          success: true,
          message: "OTP sent successfully",
          verifyId: otpRes.verifyId,
        });
      } else {
        return res
          .status(400)
          .json({ success: false, message: "Phone Number already verified" });
      }
    }

    res
      .status(400)
      .json({ success: false, message: "Something wrong, pls try again" });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

export const emailOtpVerify = async (req: Request, res: Response) => {
  try {
    const { id, otp ,update} = req.body;

    const verifyObj = await db.verifyEmail.findUnique({
      where: {
        id: id,
      },
    });

    if (!verifyObj)
      return res
        .status(400)
        .json({ success: false, message: "Verify Email Object not found" });

    if (verifyObj.otp !== parseInt(otp))
      return res.status(400).json({ success: false, message: "Invalid otp" });

    await db.verifyEmail.update({
      where: {
        id: id,
      },
      data: {
        isVerify: true,
        verifyTime: new Date(),
      },
    });
    if (update) {
      await db.users.update({
        where: {
          id: verifyObj.UserId,
        },
        data: {
          email: verifyObj.email,
        },
      });
    }
    res.status(200).json({ success: true, message: "Otp Verified" });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

export const phoneOtpVerify = async (req: Request, res: Response) => {
  try {
    const { id, otp } = req.body;

    const verifyObj = await db.verifyPhone.findUnique({
      where: {
        id: id,
      },
    });

    if (!verifyObj)
      return res.status(400).json({
        success: false,
        message: "Verify Phone Number Object not found",
      });

    if (verifyObj.otp !== parseInt(otp))
      return res.status(400).json({ success: false, message: "Invalid otp" });

    await db.verifyPhone.update({
      where: {
        id: id,
      },
      data: {
        isVerify: true,
        verifyTime: new Date(),
      },
    });

    res.status(200).json({ success: true, message: "Otp Verified" });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

// business register
// export const businessRegister = async (req: Request, res: Response) => {
//   try {
//     const { data } = req.body;

//     let errorMessage: { path: string; message: string }[] = [];

//     await Promise.allSettled(
//       data?.map(async (e: any, index: number) => {
//         try {
//           const extBusiness = await db.businessInfo.findFirst({
//             where: {
//               name: e?.name,
//               typeId: e?.typeId,
//             },
//           });
//           if (extBusiness) {
//             errorMessage.push({
//               path: `${index}.name`,
//               message: "same name business already exist",
//             });
//           }
//         } catch (err) {
//           // console.log(err);
//           errorMessage.push({
//             path: `${index}.name`,
//             message: "something wrong",
//           });
//         }
//       })
//     );

//     if (errorMessage.length > 0) {
//       return res.status(400).json({
//         success: false,
//         message: "Invalid data",
//         details: errorMessage,
//       });
//     }

//     const acceptTermsCondition = new Date();

//     const saveData = await db.$transaction(
//       data.map((e: any) =>
//         db.businessInfo.create({
//           data: {
//             name: e?.name,
//             typeId: e?.typeId,
//             userId: req.user?.userId || "",
//             address: {
//               addressLine01: e?.addressLine01,
//               addressLine02: e?.addressLine02,
//               country: e?.country,
//               state: e?.state,
//               city: e?.city,
//               zipCode: e?.zipCode,
//             },
//             email: e?.email,
//             phoneNumber: e?.phoneNumber,
//             website: e?.website,
//             logoUrl:e?.logoUrl,
//             isContactPersonSame: e?.isContactPersonSame,
//             contactPerson:
//               e?.isContactPersonSame === false ? e?.contactPerson : undefined,
//             acceptTermsCondition: acceptTermsCondition,
//             documentList:
//               e?.documentList?.length > 0
//                 ? {
//                   createMany: {
//                     data: e?.documentList?.map((doc: any) => ({
//                       userId: req.user?.userId || "",
//                       originalDocInfoId: doc?.originalDocInfoId,
//                       filename: doc?.filename,
//                       url: doc?.url,
//                     })),
//                   },
//                 }
//                 : undefined,
//           },
//         })
//       )
//     );

//     res.status(201).json({
//       success: true,
//       message: "created successfully",
//       userId: saveData,
//     });
//   } catch (error) {
//     console.error(error);
//     res.status(500).json({ message: "Internal Server Error", success: false });
//   }
// };

// Check username
export const checkUsername = async (req: Request, res: Response) => {
  try {
    const { username } = req.body;

    const findObj = await db.users.findUnique({
      where: {
        username: username.toLowerCase(),
      },
    });

    if (findObj)
      return res.status(200).json({
        success: true,
        message: "username not available",
        available: false,
      });

    res
      .status(200)
      .json({ success: true, message: "username available", available: true });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};
// Check Email
export const checkEmail = async (req: Request, res: Response) => {
  try {
    const { email } = req.body;

    const findObj = await db.users.findUnique({
      where: {
        email: email.toLowerCase(),
      },
    });

    if (findObj)
      return res.status(200).json({
        success: true,
        message: "email not available",
        available: false,
      });

    res
      .status(200)
      .json({ success: true, message: "email available", available: true });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};
// Check Phone Number
export const checkPhoneNumber = async (req: Request, res: Response) => {
  try {
    const { phoneNumber } = req.body;

    const findObj = await db.users.findFirst({
      where: {
        phoneNumber: phoneNumber
      }
    });

    if (findObj)
      return res.status(200).json({
        success: true,
        message: "phone number not available",
        available: false,
      });

    res
      .status(200)
      .json({ success: true, message: "phone number available", available: true });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};
// Check username
export const checkUserDetails = async (req: Request, res: Response) => {
  try {
    const { field,value } = req.body;

    const findObj = await db.users.findUnique({
      where: {
        username: value.toLowerCase(),
      },
    });

    if (findObj)
      return res.status(200).json({
        success: true,
        message: "username not available",
        available: false,
      });

    res
      .status(200)
      .json({ success: true, message: "username available", available: true });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};


// generate session token
export const generateSessionToken = async (req: Request, res: Response) => {
  try {
    const refreshToken = req.header("Refresh-Token");

    if (!refreshToken) {
      return res.status(403).json({ error: "Unauthorized: No token provided" });
    }

    // Refresh Token Verify
    let decodedRefreshToken: JwtPayload | null = null;
    try {
      decodedRefreshToken = jwt.verify(
        refreshToken,
        REFRESH_TOKEN_SECRET
      ) as JwtPayload;
    } catch (error) {
      if (error instanceof jwt.TokenExpiredError) {
        return res.status(403).json({ error: "Forbidden: Token expired" });
      }
      if (error instanceof jwt.JsonWebTokenError) {
        console.log(error);

        return res.status(403).json({ error: "Forbidden: Invalid token" });
      }
      // For other unexpected errors
      return res.status(500).json({ error: "Internal Server Error" });
    }

    if (!decodedRefreshToken) {
      return res
        .status(403)
        .json({ error: "Forbidden: Invalid refresh token" });
    }

    // Check if the refresh token is associated with a valid user
    const user = await db.users.findUnique({
      where: {
        id: decodedRefreshToken.userId,
        devices: {
          some: {
            refreshToken: refreshToken,
          },
        },
      },
    });

    if (!user) {
      return res
        .status(403)
        .json({ error: "Forbidden: Invalid refresh token" });
    }

    // update the last login time
    // Update the device info and ip address
    await db.deviceInfo.updateMany({
      where: {
        UserId: user.id,
        refreshToken: refreshToken,
      },
      data: {
        lastLogin: new Date(),
      },
    });

    const jwtPayload: JwtPayload = {
      userId: user.id,
      username: user.username,
      email: user.email,
      // country: user.country,
      firstName: user.firstName,
      lastName: user.lastName,
      middleName: user.middleName || "",
      imageUrl: user.imageUrl,
      role: user.role,
    };
    const sessionToken = jwt.sign(jwtPayload, JWT_SECRET, {
      expiresIn: "2h",
    });

    // if login user is Admin user then business always true
    // const isBusiness = user.businessInfo.length > 0;
    // const isBusinessFlag = user.role === "ADMIN" ? true : isBusiness;

    res.status(200).json({
      success: true,
      message: "Successful",
      sessionToken, // this token content user info
      // business: isBusinessFlag,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};


// Forgot username
export const forgotUsername = async (req: Request, res: Response) => {
  try {
    const { firstName, lastName, email } = req.body;
    const findResult = await db.users.findFirst({
      where: {
        firstName: firstName,
        lastName: lastName,
        email: email,
      },
    });
    if (!findResult) {
      return res.status(200).json({
        success: false,
        message: "No User Found!",
      });
    }
    const subject = "Forgot Username Request for SafalVendors"
    const content = `Your SafalVendors username is ${findResult.username}`;

    sendEmail(subject, email, content);

    res.status(200).json({
      success: true,
      message: "User ID sent on your E-mail address. Please check your inbox!",
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

// Forgot Password
export const forgotPassword = async (req: Request, res: Response) => {
  try {
    const { email } = req.body;
    const findResult = await db.users.findFirst({
      where: {
        email: email,
      },
    });
    if (!findResult) {
      return res.status(200).json({
        success: false,
        message: "No User Found!",
      });
    }
    const token = generateResetToken(70) ;
    await db.resetPassword.create({
      data: {
        token: token,
        UserId: findResult.id,
      }
    })

    const subject = " - forgot Password";
    const fullUrl = `${FRONT_URL}reset-password/${token}`;
    
    
    const content =`click on below URL to reset your password ${fullUrl}`;
    sendEmail(subject, email, content);

    res.status(200).json({
      success: true,
      message: "Reset Password link sent on your E-mail address. Please check your inbox!",
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};
// Reset Password
export const resetPassword = async (req: Request, res: Response) => {
  try {
    const { token, password } = req.body;

    const resetObj = await db.resetPassword.findFirst({
      where: {
        token: token,
        isReset: false,
      }
    });
    if (!resetObj)
      return res
        .status(400)
        .json({ success: false, message: "Invalid link" });

    const tokenCreationTime = new Date(resetObj.createdAt);
    const currentTime = new Date();
    const timeDifference = (currentTime.getTime() - tokenCreationTime.getTime()) / (1000 * 60); // Difference in minutes

    if (timeDifference > TOKEN_EXPIRY_MINUTES) {
      return res.status(400).json({ success: false, message: "Token has expired" });
    }

    await db.resetPassword.update({
      where: {
        id: resetObj.id
      },
      data: {
        isReset: true,
        resetTime: new Date(),
      },
    });
    const hashedPassword = await bcrypt.hash(password, 10);

    const findResult = await db.users.update({
      where: {
        id: resetObj.UserId,
      },
      data: {
        password: hashedPassword
      }
    });

    if (findResult) {
      res.status(200).json({
        success: true,
        message: "Reset Password updated successfully",
      });
    }
    else {
      res.status(401).json({
        success: false,
        message: "Something went wrong",
      });
    }
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

export const resetPasswordTokenVerify = async (req: Request, res: Response) => {
  try {
    const { token } = req.body;
    const resetObj = await db.resetPassword.findFirst({
      where: {
        token: token,
        isReset: false
      },
    });
    if (!resetObj)
      return res.status(401).json({ success: false, message: "Invalid link" });

    const tokenCreationTime = new Date(resetObj.createdAt);
    const currentTime = new Date();
    const timeDifference = (currentTime.getTime() - tokenCreationTime.getTime()) / (1000 * 60); // Difference in minutes

    if (timeDifference > TOKEN_EXPIRY_MINUTES) {
      return res.status(401).json({ success: false, message: "Token has expired" });
    }

    res.status(200).json({ success: true, message: "Link is valid" });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

function generateResetToken(length:number): string {
  const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
  let result = '';
  const charactersLength = characters.length;

  for (let i = 0; i < length; i++) {
    result += characters.charAt(Math.floor(Math.random() * charactersLength));
  }

  return result
}