import { Request, Response } from 'express';
import { PrismaClient } from '@prisma/client';
import bcrypt from 'bcrypt';
import crypto from 'crypto';

const prisma = new PrismaClient();

// Add new user
export const addUser = async (req: Request, res: Response) => {
    try {
        const { username, email, password, phoneNumber } = req.body;

        // Check if user already exists
        const existingUser = await prisma.users.findUnique({
            where: { email }
        });

        if (existingUser) {
            return res.status(400).json({ message: 'User already exists with this email' });
        }

        // Hash password
        const hashedPassword = await bcrypt.hash(password, 10);

        // Generate auth token
        const authToken = crypto.randomBytes(32).toString('hex');

        // Create new user
        const newUser = await prisma.users.create({
            data: {
                username,
                email,
                password: hashedPassword,
                phoneNumber,
                authToken,
                role: 'DEFAULT',
                firstName: username, // Required field
                lastName: '', // Required field
                gender: 'NOT_SPECIFY', // Required field
                dob: new Date(), // Required field
            }
        });

        // Remove password from response
        const { password: _, ...userWithoutPassword } = newUser;

        res.status(201).json({
            message: 'User created successfully',
            user: userWithoutPassword
        });
    } catch (error) {
        console.error('Error in addUser:', error);
        res.status(500).json({ message: 'Internal server error' });
    }
};

// Edit user
export const editUser = async (req: Request, res: Response) => {
    try {
        const { id } = req.params;
        const { username, email, phoneNumber, role } = req.body;

        // Check if user exists
        const existingUser = await prisma.users.findUnique({
            where: { id }
        });

        if (!existingUser) {
            return res.status(404).json({ message: 'User not found' });
        }

        // Update user
        const updatedUser = await prisma.users.update({
            where: { id },
            data: {
                username,
                email,
                phoneNumber,
                role: role || existingUser.role,
                firstName: username, // Update firstName as well since it's required
            }
        });

        // Remove password from response
        const { password: _, ...userWithoutPassword } = updatedUser;

        res.json({
            message: 'User updated successfully',
            user: userWithoutPassword
        });
    } catch (error) {
        console.error('Error in editUser:', error);
        res.status(500).json({ message: 'Internal server error' });
    }
};

// Delete user
export const deleteUser = async (req: Request, res: Response) => {
    try {
        const { id } = req.params;

        // Check if user exists
        const existingUser = await prisma.users.findUnique({
            where: { id }
        });

        if (!existingUser) {
            return res.status(404).json({ message: 'User not found' });
        }

        // Delete user
        await prisma.users.delete({
            where: { id }
        });

        res.json({ message: 'User deleted successfully' });
    } catch (error) {
        console.error('Error in deleteUser:', error);
        res.status(500).json({ message: 'Internal server error' });
    }
}; 