import { Country, Gender } from "@prisma/client";
import { z } from "zod";

// {
//   "username": "user12",
//   "firstName": "Hello",
//   "middleName": "", // optional
//   "lastName": "World",
//   "gender": "",
//   "email": "",
//   "phoneNumber": 9876543212, // 9876543212,
//   "dob": "",
//   "country": ""
// }

// Define a regular expression for phone number validation (10 or 11 digits)
// const phoneNumberRegex = /^\d{10,11}$/;
const phoneNumberWithCountryCodeRegex =
  /^\+?\d{1,3}?[-.\s]?\(?\d{2,4}\)?[-.\s]?\d{3,4}[-.\s]?\d{4}$/;

// Calculate the minimum date (18 years ago from today)
const minDate = new Date(
  new Date().getFullYear() - 18,
  new Date().getMonth(),
  new Date().getDate()
);

export const registrationValidation = z.object({
  username: z
    .string({ message: "username is required" })
    .min(3, { message: "username is required" })
    .toLowerCase(),
  email: z
    .string({ message: "email is required" })
    .email({ message: "invalid email format" })
    .toLowerCase(),
  password: z
    .string({ message: "password is required" })
    .min(8, { message: "password must be min 8 char" }),
  firstName: z
    .string({ message: "first name is required" })
    .min(2, { message: "first name is required" }),
  middleName: z.string().toLowerCase().optional(),
  lastName: z
    .string({ message: "last name is required" })
    .min(2, { message: "last name is required" }),
  phoneNumber: z
    .string()
    // .positive()
    .refine((value) => phoneNumberWithCountryCodeRegex.test(value.toString()), {
      message: "Invalid phone number format.",
    }),
  dob: z.string().refine(
    (dateString) => {
      const date = new Date(dateString);
      return date <= minDate;
    },
    {
      message: "Date of birth must be at least 18 years ago from today.",
    }
  ), // YYYY-MM-DD
  country: z.nativeEnum(Country),
});
export const registrationChangeValidation = z.object({
  email: z
    .string({ message: "email is required" })
    .email({ message: "invalid email format" })
    .toLowerCase(),
  phoneNumber: z
    .string()
    .refine((value) => phoneNumberWithCountryCodeRegex.test(value.toString()), {
      message: "Invalid phone number format.",
    }),
  userId: z
    .string({ message: "userId is required" })
    .min(3, { message: "userId is required" }),
});

export const loginValidation = z.object({
  email: z
    .string({ message: "email is required" })
    .email({ message: "invalid email format" })
    .toLowerCase(),
  password: z
    .string({ message: "password is required" })
    .min(8, { message: "password must be min 8 char" }),
});


export const checkUsernameValidation = z.object({
  username: z
    .string({ message: "username is required" })
    .min(3, { message: "username is required" }),
});
export const checkPhoneNumberValidation = z.object({
  phoneNumber: z
    .string({ message: "username is required" })
    .min(3, { message: "username is required" }),
});
export const checkEmalValidation = z.object({
  email: z
    .string({ message: "username is required" })
    .email()
});

export const emailVerifySendValidation = z.object({
  email: z
    .string({ message: "email is required" })
    .email({ message: "invalid email format" })
    .toLowerCase(),
  userId: z
    .string({ message: "userId is required" })
    .min(3, { message: "userId is required" }),
  type: z.enum(["NEW", "CHANGE"]),
});
export const phoneVerifySendValidation = z.object({
  phoneNumber: z
    .string()
    // .positive()
    .refine((value) => phoneNumberWithCountryCodeRegex.test(value.toString()), {
      message: "Invalid phone number format.",
    }),
  userId: z
    .string({ message: "userId is required" })
    .min(3, { message: "userId is required" }),
  type: z.enum(["NEW", "CHANGE"]),
});

export const otpVerifyValidation = z.object({
  otp: z.number({ message: "OTP is required" }).min(100000).max(999999),
  id: z
    .string({ message: "id is required" })
    .min(3, { message: "id is required" }),
});

const businessRegistrationValidationDocumentList = z.object({
  originalDocInfoId: z
    .string({ message: "original Doc Info Id is required" })
    .min(2, { message: "original Doc Info Id is required" }),
  filename: z
    .string({ message: "filename is required" })
    .min(2, { message: "filename is required" }),
  url: z
    .string({ message: "invalid url format" })
    .url({ message: "invalid url format" }),
});

export const businessRegistrationValidation = z.object({
  name: z
    .string({ message: "name is required" })
    .min(2, { message: "name is required" }),
  typeId: z
    .string({ message: "type is required" })
    .min(3, { message: "type is required" }),
  addressLine01: z
    .string({ message: "address line 1 is required" })
    .min(2, { message: "address line 1 is required" }),
  addressLine02: z.string({ message: "address line 2 is required" }).optional(),
  country: z.nativeEnum(Country),
  state: z
    .string({ message: "state is required" })
    .min(1, { message: "state is required" }),
  city: z
    .string({ message: "city is required" })
    .min(1, { message: "city is required" }),
  zipCode: z.number({ message: "zipCode is required" }),
  email: z
    .string({ message: "email is required" })
    .email({ message: "invalid email format" })
    .toLowerCase(),
  phoneNumber: z
    .string()
    .refine((value) => phoneNumberWithCountryCodeRegex.test(value.toString()), {
      message: "Invalid phone number format.",
    }),
  website: z
    .string({ message: "website is required" }),
  isContactPersonSame: z.boolean({ message: "invalid format" }),
  contactPerson: z
    .object({
      firstName: z
        .string({ message: "first name is required" })
        .min(2, { message: "last name is required" }),
      middleName: z.string().toLowerCase().optional(),
      lastName: z
        .string({ message: "last name is required" })
        .min(2, { message: "last name is required" }),
      email: z
        .string({ message: "email is required" })
        .email({ message: "invalid email format" })
        .toLowerCase(),
      phoneNumber: z
        .string()
        .refine(
          (value) => phoneNumberWithCountryCodeRegex.test(value.toString()),
          {
            message: "Invalid phone number format.",
          }
        ),
    })
    .optional(),
  documentList: z.array(businessRegistrationValidationDocumentList),
});

// Define the schema for multiple business registrations
export const multipleBusinessRegistrationsValidation = z.object({
  data: z.array(businessRegistrationValidation),
});


export const forgotUsernameValidation = z.object({
  email: z
    .string({ message: "email is required" })
    .email({ message: "invalid email format" }),
  firstName: z
    .string({ message: "First Name is required" })
    .min(3, { message: "First Name is required" }),
  lastName: z
    .string({ message: "Last Name is required" })
    .min(3, { message: "Last Name is required" }),
});
export const forgotPasswordValidation = z.object({
  email: z
    .string({ message: "email is required" })
    .email({ message: "invalid email format" }),
});
export const resetPasswordValidation = z.object({
  token: z
    .string({ message: "token is required" }),
  password: z
    .string({ message: "password is required" })
    .min(8, { message: "password must be min 8 char" }),
});
export const resetPasswordTokenVerifyValidation = z.object({
  token: z
    .string({ message: "token is required" }),
});